@@ -29,7 +29,7 @@ use {
     },
     solana_pubkey::Pubkey,
     solana_runtime::bank::Bank,
-    solana_streamer::streamer::{self, PacketBatchReceiver, StreamerReceiveStats},
+    solana_streamer::streamer::{PacketBatchReceiver, StreamerReceiveStats},
     solana_time_utils::timestamp,
     std::{
         collections::HashSet,
@@ -159,20 +159,24 @@ impl AncestorHashesService {
     ) -> Self {
         let outstanding_requests = Arc::<RwLock<OutstandingAncestorHashesRepairs>>::default();
         let (response_sender, response_receiver) = unbounded();
-        let t_receiver = streamer::receiver(
-            "solRcvrAncHash".to_string(),
-            ancestor_hashes_request_socket.clone(),
-            exit.clone(),
-            response_sender.clone(),
-            Recycler::default(),
-            Arc::new(StreamerReceiveStats::new(
-                "ancestor_hashes_response_receiver",
-            )),
-            Some(Duration::from_millis(1)), // coalesce
-            false,                          // use_pinned_memory
-            None,                           // in_vote_only_mode
-            false,                          // is_staked_service
-        );
+
+        // FOGO: Use net_connector to receive ancestor hash packets from XDP tile
+        // instead of the traditional UDP socket receiver.
+        let t_receiver =
+            fogo_net_connector::receiver::<{ fogo_net_connector::protocol::ANCESTOR_HASHES }>(
+                "solRcvrAncHash".to_string(),
+                ancestor_hashes_request_socket.clone(),
+                exit.clone(),
+                response_sender.clone(),
+                Recycler::default(),
+                Arc::new(StreamerReceiveStats::new(
+                    "ancestor_hashes_response_receiver",
+                )),
+                Some(Duration::from_millis(1)), // coalesce
+                false,                          // use_pinned_memory
+                None,                           // in_vote_only_mode
+                false,                          // is_staked_service
+            );
 
         let AncestorHashesChannels {
             ancestor_hashes_request_quic_sender,
