@@ -354,7 +354,8 @@ pub(crate) struct RepairPeers {
 struct Node {
     pubkey: Pubkey,
     serve_repair: SocketAddr,
-    serve_repair_quic: SocketAddr,
+    // FOGO: Quic repair not used
+    /*serve_repair_quic: SocketAddr,*/
 }
 
 impl RepairPeers {
@@ -374,7 +375,7 @@ impl RepairPeers {
                 let node = Node {
                     pubkey: *peer.pubkey(),
                     serve_repair: peer.serve_repair(Protocol::UDP)?,
-                    serve_repair_quic: peer.serve_repair(Protocol::QUIC)?,
+                    // FOGO: quic repair not used, serve_repair_quic: peer.serve_repair(Protocol::QUIC)?,
                 };
                 Some((node, weight))
             })
@@ -1110,8 +1111,8 @@ impl ServeRepair {
         repair_validators: &Option<HashSet<Pubkey>>,
         outstanding_requests: &mut OutstandingShredRepairs,
         identity_keypair: &Keypair,
-        repair_request_quic_sender: &AsyncSender<(SocketAddr, Bytes)>,
-        repair_protocol: Protocol,
+        _repair_request_quic_sender: &AsyncSender<(SocketAddr, Bytes)>,
+        _repair_protocol: Protocol,
     ) -> Result<Option<(SocketAddr, Vec<u8>)>> {
         // find a peer that appears to be accepting replication and has the desired slot, as indicated
         // by a valid tvu port location
@@ -1127,7 +1128,8 @@ impl ServeRepair {
                     .unwrap_or_default()
                     > 0;
                 let repair_peers =
-                    RepairPeers::new(Instant::now(), &repair_peers, &weights, staked_validator)?;
+                    RepairPeers::new(Instant::now(), &repair_peers, &weights, staked_validator)
+                        .inspect_err(|err| error!("fd_net repair peers error: {:?}", err))?;
                 peers_cache.put(slot, repair_peers);
                 peers_cache.get(&slot).unwrap()
             }
@@ -1147,15 +1149,19 @@ impl ServeRepair {
             peer.pubkey,
             repair_request
         );
-        match repair_protocol {
+        Ok(Some((peer.serve_repair, out)))
+
+        // FOGO: Quic repair not used
+        /*match repair_protocol {
             Protocol::UDP => Ok(Some((peer.serve_repair, out))),
             Protocol::QUIC => {
+                // FOGO: Qui
                 repair_request_quic_sender
                     .blocking_send((peer.serve_repair_quic, Bytes::from(out)))
                     .map_err(|_| Error::SendError)?;
                 Ok(None)
             }
-        }
+        }*/
     }
 
     pub(crate) fn repair_request_ancestor_hashes_sample_peers(
