@@ -77,6 +77,7 @@ pub struct CostTracker {
     in_flight_transaction_count: Saturating<usize>,
     secp256r1_instruction_signature_count: Saturating<u64>,
     block_txn_count_limit: u64,
+    block_txn_byte_limit: u64,
 }
 
 impl Default for CostTracker {
@@ -117,6 +118,7 @@ impl Default for CostTracker {
             in_flight_transaction_count: Saturating(0),
             secp256r1_instruction_signature_count: Saturating(0),
             block_txn_count_limit: u64::MAX,
+            block_txn_byte_limit: u64::MAX,
         }
     }
 }
@@ -130,6 +132,7 @@ impl CostTracker {
             self.vote_cost_limit,
         );
         new.block_txn_count_limit = self.block_txn_count_limit;
+        new.block_txn_byte_limit = self.block_txn_byte_limit;
         new
     }
 
@@ -145,6 +148,7 @@ impl CostTracker {
         self.secp256r1_instruction_signature_count = Saturating(0);
         self.in_flight_transaction_count = Saturating(0);
         self.block_txn_count_limit = u64::MAX;
+        self.block_txn_byte_limit = u64::MAX;
     }
 
     /// Get the overall block limit.
@@ -161,6 +165,10 @@ impl CostTracker {
         self.block_txn_count_limit
     }
 
+    pub fn get_block_txn_byte_limit(&self) -> u64 {
+        self.block_txn_byte_limit
+    }
+
     /// allows to adjust the account cost limit
     pub fn set_account_cost_limit(&mut self, account_cost_limit: u64) {
         self.account_cost_limit = account_cost_limit;
@@ -182,6 +190,10 @@ impl CostTracker {
         self.block_txn_count_limit = limit;
     }
 
+    pub fn set_block_txn_byte_limit(&mut self, limit: u64) {
+        self.block_txn_byte_limit = limit;
+    }
+
     pub fn set_block_cost_limit(&mut self, limit: u64) {
         self.block_cost_limit = limit;
     }
