@@ -160,6 +160,7 @@ impl FeatureSet {
             reenable_zk_elgamal_proof_program: self
                 .is_active(&reenable_zk_elgamal_proof_program::id()),
             enable_lockout_v11: self.is_active(&enable_lockout_v11::id()),
+            enable_session_wrap: self.is_active(&enable_session_wrap::id()),
         }
     }
 }
@@ -1181,10 +1182,22 @@ pub mod fixed_2pct_inflation {
     solana_pubkey::declare_id!("ByxE6PGEvJQCW4ByXyQgYSgXxxjxfocbAZAiJbuq9AiB");
 }
 
+pub mod reduce_leader_slots {
+    solana_pubkey::declare_id!("7t213exR4ku6pD4J9TMhzxKu4MiUDe2HJFN8BB5577pB");
+}
+
 pub mod cu_increase_block_lvl_txn_payload_limit {
     solana_pubkey::declare_id!("CQEFNV88L35tbEX2YtMtagqKVsqGwcJdeyDfkMC1DPuK");
 }
 
+pub mod block_lvl_txn_payload_limit {
+    solana_pubkey::declare_id!("3Kx6GJBjDsuxPsj2GkHbkPzC9WWt8YbFvABAbiD7r2kR");
+}
+
+pub mod enable_session_wrap {
+    solana_pubkey::declare_id!("9ySzykRH1PDGYLoBCp3bAvDLfz4mY6ZpmE6tofWaiD7P");
+}
+
 pub static FEATURE_NAMES: LazyLock<AHashMap<Pubkey, &'static str>> = LazyLock::new(|| {
     [
         (secp256k1_program_enabled::id(), "secp256k1 program"),
@@ -1440,7 +1453,10 @@ pub static FEATURE_NAMES: LazyLock<AHashMap<Pubkey, &'static str>> = LazyLock::n
         (raise_writeable_account_units_2_5::id(), "Raise max writeable account units to 2.5M CU"),
         (deprecate_secondary_zone_accounts::id(), "Deprecate secondary zone accounts"),
         (fixed_2pct_inflation::id(), "Fixed 2% inflation"),
+        (reduce_leader_slots::id(), "Reduce leader slots"),
         (cu_increase_block_lvl_txn_payload_limit::id(), "Increase CU limit to 10 million. Set txns allowed in block to 4000"),
+        (block_lvl_txn_payload_limit::id(), "Set txn payload limit per block to 2 million bytes"),
+        (enable_session_wrap::id(), "Enables the session wrap and the intent transfer system program instructions"),
     ]
     .iter()
     .cloned()
