@@ -22,7 +22,7 @@ const MAX_SCHEDULES: usize = 10;
 // FIREDANCER: Some constants for the number and size of the leader
 // schedules we send across the IPC boundary.
 const FIREDANCER_STAKE_WEIGHT_CNT: usize = 40_200;
-const FIREDANCER_PACKET_HEADER_SZ: usize = 48;
+const FIREDANCER_PACKET_HEADER_SZ: usize = 56;
 const FIREDANCER_PACKET_RECORD_SZ: usize = 72;
 const FIREDANCER_PACKET_MAX_SZ: usize =
     FIREDANCER_PACKET_HEADER_SZ + FIREDANCER_STAKE_WEIGHT_CNT * FIREDANCER_PACKET_RECORD_SZ;
@@ -333,6 +333,9 @@ impl LeaderScheduleCache {
             memory[24..32].copy_from_slice(&slot_cnt.to_le_bytes());
             memory[32..40].copy_from_slice(&remaining_stake.to_le_bytes());
             memory[40..48].copy_from_slice(&use_vote_keyed_leader_schedule.to_le_bytes());
+            let leader_slots = bank.get_leader_slots(epoch);
+            memory[48..56].copy_from_slice(&leader_slots.to_le_bytes());
+
             for (i, (vote, id, stake)) in stakes.iter().enumerate() {
                 let offset = FIREDANCER_PACKET_HEADER_SZ + i * FIREDANCER_PACKET_RECORD_SZ;
                 memory[offset..offset + 32].copy_from_slice(&vote.to_bytes());
