@@ -198,6 +198,30 @@ pub fn parse_system(
                 }),
             })
         }
+        SystemInstruction::SessionWrap { lamports } => {
+            check_num_system_accounts(&instruction.accounts, 3)?;
+            Ok(ParsedInstructionEnum {
+                instruction_type: "sessionWrap".to_string(),
+                info: json!({
+                    "source": account_keys[instruction.accounts[0] as usize].to_string(),
+                    "destination": account_keys[instruction.accounts[1] as usize].to_string(),
+                    "session": account_keys[instruction.accounts[2] as usize].to_string(),
+                    "lamports": lamports,
+                }),
+            })
+        }
+        SystemInstruction::IntentTransfer { lamports } => {
+            check_num_system_accounts(&instruction.accounts, 3)?;
+            Ok(ParsedInstructionEnum {
+                instruction_type: "intentTransfer".to_string(),
+                info: json!({
+                    "source": account_keys[instruction.accounts[0] as usize].to_string(),
+                    "destination": account_keys[instruction.accounts[1] as usize].to_string(),
+                    "intentTransferSetter": account_keys[instruction.accounts[2] as usize].to_string(),
+                    "lamports": lamports,
+                }),
+            })
+        }
     }
 }
 
