@@ -77,11 +77,14 @@ fn verify_reachable_ports(
             .map(|addr| socket_addr_space.check(addr))
             .unwrap_or_default()
     };
-    let mut udp_sockets = vec![&node.sockets.gossip, &node.sockets.repair];
+    // Gossip and repair are handled throygh netcon, and can't be verified here, because their respective
+    // UDP sockets bound on the `Node`, dont have the right working sockets, due to xdp re-routing the data for them.
+    let mut udp_sockets = vec![/*&node.sockets.gossip*/ &node.sockets.repair];
 
-    if verify_address(&node.info.serve_repair(Protocol::UDP)) {
+    // serve repair socket is also handled by xdp.
+    /*if verify_address(&node.info.serve_repair(Protocol::UDP)) {
         udp_sockets.push(&node.sockets.serve_repair);
-    }
+    }*/
     if verify_address(&node.info.tpu(Protocol::UDP)) {
         udp_sockets.extend(node.sockets.tpu.iter());
         udp_sockets.extend(&node.sockets.tpu_quic);
@@ -120,10 +123,11 @@ fn verify_reachable_ports(
         }
     }
 
-    if let Some(ip_echo) = &node.sockets.ip_echo {
+    // we don't need to check our own ip echo.
+    /*if let Some(ip_echo) = &node.sockets.ip_echo {
         let ip_echo = ip_echo.try_clone().expect("unable to clone tcp_listener");
         tcp_listeners.push(ip_echo);
-    }
+    }*/
 
     solana_net_utils::verify_all_reachable_tcp(&cluster_entrypoint.gossip().unwrap(), tcp_listeners)
 }
