@@ -342,6 +342,7 @@ pub fn add_args<'a>(app: App<'a, 'a>, default_args: &'a DefaultArgs) -> App<'a,
             .long("gossip-port")
             .value_name("PORT")
             .takes_value(true)
+            .required(true)
             .help("Gossip port number for the validator"),
     )
     .arg(
@@ -1692,6 +1693,22 @@ pub fn add_args<'a>(app: App<'a, 'a>, default_args: &'a DefaultArgs) -> App<'a,
             .validator(is_parsable::<u16>)
             .help("Port to use for receiving transactions in the TVU."),
     )
+    // Fogo: Serve repair port is handled by netcon
+    .arg(
+        Arg::with_name("firedancer_serve_repair_port")
+            .long("firedancer-serve-repair-port")
+            .takes_value(true)
+            .validator(is_parsable::<u16>)
+            .help("Port to use for receiving repair requests."),
+    )
+    // Fogo: Ancestor hashes port is handled by netcon
+    .arg(
+        Arg::with_name("firedancer_ancestor_hashes_port")
+            .long("firedancer-ancestor-hashes-port")
+            .takes_value(true)
+            .validator(is_parsable::<u16>)
+            .help("Port to use for receiving ancestor hashes responses."),
+    )
     .arg(
         Arg::with_name("fogo_config")
             .long("fogo-config")
