@@ -277,7 +277,8 @@ pub fn execute(
     };
 
     let private_rpc = matches.is_present("private_rpc");
-    let do_port_check = !matches.is_present("no_port_check");
+    // FOGO: Ip echo server is not supported
+    let do_port_check = false;
     let tpu_coalesce = value_t!(matches, "tpu_coalesce_ms", u64)
         .map(Duration::from_millis)
         .unwrap_or(DEFAULT_TPU_COALESCE);
@@ -454,13 +455,12 @@ pub fn execute(
             Err(format!("invalid entrypoint address: {addr}"))?;
         }
     }
-    // TODO: Once entrypoints are updated to return shred-version, this should
-    // abort if it fails to obtain a shred-version, so that nodes always join
+    // Abort if it fails to obtain a shred-version, so that nodes always join
     // gossip with a valid shred-version. The code to adopt entrypoint shred
     // version can then be deleted from gossip and get_rpc_node above.
-    let expected_shred_version = value_t!(matches, "expected_shred_version", u16)
-        .ok()
-        .or_else(|| get_cluster_shred_version(&entrypoint_addrs, bind_address));
+    let expected_shred_version = value_t!(matches, "expected_shred_version", u16).expect(
+        "failed to resolve shred version, either from the config or from the cluster entrypoints",
+    );
 
     let tower_storage: Arc<dyn tower_storage::TowerStorage> =
         match value_t_or_exit!(matches, "tower_storage", String).as_str() {
@@ -731,7 +731,7 @@ pub fn execute(
         expected_bank_hash: matches
             .value_of("expected_bank_hash")
             .map(|s| Hash::from_str(s).unwrap()),
-        expected_shred_version,
+        expected_shred_version: Some(expected_shred_version),
         new_hard_forks: hardforks_of(matches, "hard_forks"),
         rpc_config: JsonRpcConfig {
             enable_rpc_transaction_history: matches.is_present("enable_rpc_transaction_history"),
@@ -1239,10 +1239,8 @@ pub fn execute(
             }
         })
         .ok_or_else(|| "unable to determine the validator's public IP address".to_string())?;
-    let gossip_port = value_t!(matches, "gossip_port", u16).or_else(|_| {
-        solana_net_utils::find_available_port_in_range(bind_address, (0, 1))
-            .map_err(|err| format!("unable to find an available gossip port: {err}"))
-    })?;
+    let gossip_port = value_t!(matches, "gossip_port", u16)
+        .or_else(|_| Err("Gossip port must be specified".to_string()))?;
     let gossip_addr = SocketAddr::new(gossip_host, gossip_port);
 
     let public_tpu_addr = matches
@@ -1311,14 +1309,16 @@ pub fn execute(
     // FIREDANCER: Get application ports from the CLI.
     let firedancer_tpu_port = value_t_or_exit!(matches, "firedancer_tpu_port", u16);
     let firedancer_tvu_port = value_t_or_exit!(matches, "firedancer_tvu_port", u16);
+    let firedancer_serve_repair_port =
+        value_t_or_exit!(matches, "firedancer_serve_repair_port", u16);
+    let firedancer_ancestor_hashes_port =
+        value_t_or_exit!(matches, "firedancer_ancestor_hashes_port", u16);
 
     // FIREDANCER: Send shred version that we retrieved from the command line or the entrypoint above to Firedancer
-    if let Some(shred_version) = expected_shred_version {
-        extern "C" {
-            fn fd_ext_shred_set_shred_version(shred_version: u64);
-        }
-        unsafe { fd_ext_shred_set_shred_version(shred_version as u64) };
+    extern "C" {
+        fn fd_ext_shred_set_shred_version(shred_version: u64);
     }
+    unsafe { fd_ext_shred_set_shred_version(expected_shred_version as u64) };
 
     let mut node = Node::new_with_external_ip(
         &identity_keypair.pubkey(),
@@ -1329,6 +1329,9 @@ pub fn execute(
         // FIREDANCER: Desired port for the TVU is passed in from the config file, so it
         // can be broadcast correctly via. gossip.
         firedancer_tvu_port,
+        // Serve repair port too
+        firedancer_serve_repair_port,
+        firedancer_ancestor_hashes_port,
     );
 
     if restricted_repair_only_mode {
@@ -1520,6 +1523,7 @@ fn validators_set(
     }
 }
 
+/* FOGO: IP echo server fetching of the shred version is not supported
 fn get_cluster_shred_version(entrypoints: &[SocketAddr], bind_address: IpAddr) -> Option<u16> {
     let entrypoints = {
         let mut index: Vec<_> = (0..entrypoints.len()).collect();
@@ -1540,7 +1544,7 @@ fn get_cluster_shred_version(entrypoints: &[SocketAddr], bind_address: IpAddr) -
         }
     }
     None
-}
+}*/
 
 fn configure_banking_trace_dir_byte_limit(
     validator_config: &mut ValidatorConfig,
