@@ -1,3 +1,5 @@
+#include <netdb.h>
+
 #include "../shared/fd_config.h"
 
 #include "../../disco/net/fd_net_tile.h"
@@ -30,6 +32,30 @@ parse_ip_port( const char * name, const char * ip_port, fd_topo_ip_port_t *parse
     FD_LOG_ERR(( "could not parse port %s in [%s]", ip_end+1, name ));
 }
 
+static int
+resolve_address( char const * address,
+                 uint       * ip_addr ) {
+  struct addrinfo hints = { .ai_family = AF_INET };
+  struct addrinfo * res;
+  int err = getaddrinfo( address, NULL, &hints, &res );
+  if( FD_UNLIKELY( err ) ) {
+    FD_LOG_WARNING(( "cannot resolve address \"%s\": %i-%s", address, err, gai_strerror( err ) ));
+    return 0;
+  }
+
+  int resolved = 0;
+  for( struct addrinfo * cur=res; cur; cur=cur->ai_next ) {
+    if( FD_UNLIKELY( cur->ai_addr->sa_family!=AF_INET ) ) continue;
+    struct sockaddr_in const * addr = (struct sockaddr_in const *)cur->ai_addr;
+    *ip_addr = addr->sin_addr.s_addr;
+    resolved = 1;
+    break;
+  }
+
+  freeaddrinfo( res );
+  return resolved;
+}
+
 void
 fd_topo_initialize( config_t * config ) {
   ulong net_tile_cnt    = config->layout.net_tile_count;
@@ -57,6 +83,7 @@ fd_topo_initialize( config_t * config ) {
   fd_topob_wksp( topo, "bank_poh"     );
   fd_topob_wksp( topo, "bank_busy"    );
   fd_topob_wksp( topo, "poh_shred"    );
+  fd_topob_wksp( topo, "poh_net_crds"    );
   fd_topob_wksp( topo, "gossip_dedup" );
   fd_topob_wksp( topo, "shred_store"  );
   fd_topob_wksp( topo, "stake_out"    );
@@ -77,6 +104,7 @@ fd_topo_initialize( config_t * config ) {
   fd_topob_wksp( topo, "sign"         );
   fd_topob_wksp( topo, "metric"       );
   fd_topob_wksp( topo, "cswtch"       );
+  fd_topob_wksp( topo, "netcon"      );
 
   #define FOR(cnt) for( ulong i=0UL; i<cnt; i++ )
 
@@ -99,7 +127,8 @@ fd_topo_initialize( config_t * config ) {
   FOR(bank_tile_cnt)   fd_topob_link( topo, "bank_pack",    "bank_pack",    16384UL,                                  USHORT_MAX,             3UL );
   /**/                 fd_topob_link( topo, "poh_pack",     "bank_poh",     128UL,                                    sizeof(fd_became_leader_t), 1UL );
   /**/                 fd_topob_link( topo, "poh_shred",    "poh_shred",    16384UL,                                  USHORT_MAX,             2UL );
-  /**/                 fd_topob_link( topo, "crds_shred",   "poh_shred",    128UL,                                    8UL  + 40200UL * 38UL,  1UL );
+  /**/                 fd_topob_link( topo, "poh_net_crds",    "poh_net_crds",    128UL,                                  9UL  + 40200UL * 46UL,             1UL );
+  /**/                 fd_topob_link( topo, "crds_shred",   "poh_shred",    128UL,                                    9UL  + 40200UL * 46UL,  1UL );
   /**/                 fd_topob_link( topo, "replay_resol", "bank_poh",     128UL,                                    sizeof(fd_completed_bank_t), 1UL );
   /**/                 fd_topob_link( topo, "executed_txn", "executed_txn", 16384UL,                                  64UL, 1UL );
   /* See long comment in fd_shred.c for an explanation about the size of this dcache. */
@@ -137,8 +166,9 @@ fd_topo_initialize( config_t * config ) {
 
   fd_topos_net_tiles( topo, config->layout.net_tile_count, &config->net, config->tiles.netlink.max_routes, config->tiles.netlink.max_peer_routes, config->tiles.netlink.max_neighbors, tile_to_cpu );
 
-  FOR(net_tile_cnt) fd_topos_net_rx_link( topo, "net_quic",  i, config->net.ingress_buffer_size );
-  FOR(net_tile_cnt) fd_topos_net_rx_link( topo, "net_shred", i, config->net.ingress_buffer_size );
+  FOR(net_tile_cnt) fd_topos_net_rx_link( topo, "net_quic",    i, config->net.ingress_buffer_size );
+  FOR(net_tile_cnt) fd_topos_net_rx_link( topo, "net_shred",   i, config->net.ingress_buffer_size );
+  FOR(net_tile_cnt) fd_topos_net_rx_link( topo, "net_netcon", i, config->net.ingress_buffer_size  );
 
   /*                                  topo, tile_name, tile_wksp, metrics_wksp, cpu_idx,                       is_agave, uses_keyswitch */
   FOR(quic_tile_cnt)   fd_topob_tile( topo, "quic",    "quic",    "metric_in",  tile_to_cpu[ topo->tile_cnt ], 0,        0 );
@@ -153,13 +183,18 @@ fd_topo_initialize( config_t * config ) {
   /**/                 fd_topob_tile( topo, "sign",    "sign",    "metric_in",  tile_to_cpu[ topo->tile_cnt ], 0,        1 );
   /**/                 fd_topob_tile( topo, "metric",  "metric",  "metric_in",  tile_to_cpu[ topo->tile_cnt ], 0,        0 );
   /**/                 fd_topob_tile( topo, "cswtch",  "cswtch",  "metric_in",  tile_to_cpu[ topo->tile_cnt ], 0,        0 );
+  /**/                 fd_topob_tile( topo, "netcon", "netcon", "metric_in",  tile_to_cpu[ topo->tile_cnt ], 1,        0 );
 
   /*                                      topo, tile_name, tile_kind_id, fseq_wksp,   link_name,      link_kind_id, reliable,            polled */
   for( ulong j=0UL; j<quic_tile_cnt; j++ )
                    fd_topos_tile_in_net(  topo,                          "metric_in", "quic_net",     j,            FD_TOPOB_UNRELIABLE, FD_TOPOB_POLLED ); /* No reliable consumers of networking fragments, may be dropped or overrun */
   for( ulong j=0UL; j<shred_tile_cnt; j++ )
                    fd_topos_tile_in_net(  topo,                          "metric_in", "shred_net",    j,            FD_TOPOB_UNRELIABLE, FD_TOPOB_POLLED ); /* No reliable consumers of networking fragments, may be dropped or overrun */
-
+  /* poh net */
+                       fd_topos_tile_in_net(  topo,                          "metric_in", "poh_net_crds",    0,            FD_TOPOB_UNRELIABLE, FD_TOPOB_POLLED ); /* Net crds consumer */
+  /* netcon reads from all net tiles */
+  for( ulong j=0UL; j<net_tile_cnt; j++ )
+                       fd_topob_tile_in(      topo, "netcon", 0UL,          "metric_in", "net_netcon",  j,            FD_TOPOB_UNRELIABLE, FD_TOPOB_POLLED ); /* No reliable consumers of networking fragments, may be dropped or overrun */
   FOR(quic_tile_cnt) for( ulong j=0UL; j<net_tile_cnt; j++ )
                        fd_topob_tile_in(  topo, "quic",    i,            "metric_in", "net_quic",     j,            FD_TOPOB_UNRELIABLE, FD_TOPOB_POLLED ); /* No reliable consumers of networking fragments, may be dropped or overrun */
   FOR(quic_tile_cnt)   fd_topob_tile_out( topo, "quic",    i,                         "quic_verify",  i                                                  );
@@ -228,6 +263,7 @@ fd_topo_initialize( config_t * config ) {
   /**/                 fd_topob_tile_out( topo, "poh",    0UL,                        "gossip_dedup", 0UL                                                  );
   /**/                 fd_topob_tile_out( topo, "poh",    0UL,                        "stake_out",    0UL                                                  );
   /**/                 fd_topob_tile_out( topo, "poh",    0UL,                        "crds_shred",   0UL                                                  );
+  /**/                 fd_topob_tile_out( topo, "poh",    0UL,                        "poh_net_crds",      0UL                                                  );
   /**/                 fd_topob_tile_out( topo, "poh",    0UL,                        "replay_resol", 0UL                                                  );
   /**/                 fd_topob_tile_out( topo, "poh",    0UL,                        "executed_txn", 0UL                                                  );
 
@@ -388,6 +424,14 @@ fd_topo_initialize( config_t * config ) {
       tile->net.shred_listen_port              = config->tiles.shred.shred_listen_port;
       tile->net.quic_transaction_listen_port   = config->tiles.quic.quic_transaction_listen_port;
       tile->net.legacy_transaction_listen_port = config->tiles.quic.regular_transaction_listen_port;
+      /* netcon */
+      tile->net.gossip_listen_port = config->gossip.port;
+      tile->net.repair_serve_listen_port = config->frankendancer.repair.serve_repair_port;
+      tile->net.repair_ancestor_hashes_listen_port = config->frankendancer.repair.ancestor_hashes_port;
+
+      if( FD_UNLIKELY( strcmp( config->gossip.host, "" ) ) )
+        if( !resolve_address( config->gossip.host, &tile->net.host_ip_addr ) )
+          FD_LOG_ERR(( "could not resolve [gossip.host] %s", config->gossip.host ));
 
     } else if( FD_UNLIKELY( !strcmp( tile->name, "netlnk" ) ) ) {
 
@@ -512,6 +556,8 @@ fd_topo_initialize( config_t * config ) {
       tile->gui.schedule_strategy         = config->tiles.pack.schedule_strategy_enum;
     } else if( FD_UNLIKELY( !strcmp( tile->name, "plugin" ) ) ) {
 
+    } else if( FD_UNLIKELY( !strcmp( tile->name, "netcon" ) ) ) {
+
     } else {
       FD_LOG_ERR(( "unknown tile name %lu `%s`", i, tile->name ));
     }
