@@ -172,6 +172,27 @@ fd_config_fillh( fd_config_t * config ) {
                  "This must be outside the dynamic port range `%s`",
                  config->tiles.shred.shred_listen_port,
                  config->frankendancer.dynamic_port_range ));
+
+  if( FD_UNLIKELY( config->gossip.port >= agave_port_min &&
+                 config->gossip.port < agave_port_max ) )
+    FD_LOG_ERR(( "configuration specifies invalid [gossip.port] `%hu`. "
+                 "This must be outside the dynamic port range `%s`",
+                 config->gossip.port,
+                 config->frankendancer.dynamic_port_range ));
+
+  if( FD_UNLIKELY( config->frankendancer.repair.serve_repair_port >= agave_port_min &&
+               config->frankendancer.repair.serve_repair_port < agave_port_max ) )
+    FD_LOG_ERR(( "configuration specifies invalid [repair.serve_repair_port] `%hu`. "
+                 "This must be outside the dynamic port range `%s`",
+                 config->frankendancer.repair.serve_repair_port,
+                 config->frankendancer.dynamic_port_range ));
+
+  if( FD_UNLIKELY( config->frankendancer.repair.ancestor_hashes_port >= agave_port_min &&
+             config->frankendancer.repair.ancestor_hashes_port < agave_port_max ) )
+    FD_LOG_ERR(( "configuration specifies invalid [repair.ancestor_hashes_port] `%hu`. "
+                 "This must be outside the dynamic port range `%s`",
+                 config->frankendancer.repair.ancestor_hashes_port,
+                 config->frankendancer.dynamic_port_range ));
 }
 
 static void
