@@ -39,6 +39,9 @@ fd_config_extract_podh( uchar *        pod,
 
   CFG_POP      ( bool,   gossip.port_check                                );
 
+  CFG_POP      ( ushort, repair.serve_repair_port                         );
+  CFG_POP      ( ushort, repair.ancestor_hashes_port                      );
+
   CFG_POP      ( bool,   consensus.snapshot_fetch                         );
   CFG_POP      ( bool,   consensus.genesis_fetch                          );
   CFG_POP      ( bool,   consensus.poh_speed_test                         );
@@ -180,6 +183,8 @@ fd_config_extract_pod( uchar *       pod,
   CFG_POP      ( uint,   net.xdp.xdp_rx_queue_size                        );
   CFG_POP      ( uint,   net.xdp.xdp_tx_queue_size                        );
   CFG_POP      ( uint,   net.xdp.flush_timeout_micros                     );
+  CFG_POP      ( uint,   net.xdp.unstaked_packet_rate_limit                 );
+  CFG_POP      ( uint,   net.xdp.staked_ip_packet_rate_limit                     );
   CFG_POP      ( uint,   net.socket.receive_buffer_size                   );
   CFG_POP      ( uint,   net.socket.send_buffer_size                      );
 
@@ -348,6 +353,8 @@ fd_config_extract_pod( uchar *       pod,
   CFG_RENAMED( tiles.net.xdp_rx_queue_size,    net.xdp.xdp_rx_queue_size    );
   CFG_RENAMED( tiles.net.xdp_tx_queue_size,    net.xdp.xdp_tx_queue_size    );
   CFG_RENAMED( tiles.net.flush_timeout_micros, net.xdp.flush_timeout_micros );
+  CFG_RENAMED( tiles.net.unstaked_packet_rate_limit, net.xdp.unstaked_packet_rate_limit );
+  CFG_RENAMED( tiles.net.staked_ip_packet_rate_limit, net.xdp.staked_ip_packet_rate_limit );
   CFG_RENAMED( tiles.net.send_buffer_size,     net.ingress_buffer_size      );
 
   CFG_RENAMED( development.net.provider,                 net.provider                   );
