@@ -1075,6 +1075,7 @@ fd_gui_handle_leader_schedule( fd_gui_t *    gui,
   ulong slot_cnt            = msg[ 3 ];
   ulong excluded_stake      = msg[ 4 ];
   ulong vote_keyed_lsched   = msg[ 5 ];
+  ulong leader_slots        = msg[ 6 ];
 
   FD_TEST( staked_cnt<=MAX_STAKED_LEADERS );
   FD_TEST( slot_cnt<=MAX_SLOTS_PER_EPOCH );
@@ -1089,7 +1090,7 @@ fd_gui_handle_leader_schedule( fd_gui_t *    gui,
   gui->epoch.epochs[ idx ].my_total_slots   = 0UL;
   gui->epoch.epochs[ idx ].my_skipped_slots = 0UL;
 
-  fd_vote_stake_weight_t const * stake_weights = fd_type_pun_const( msg+6UL );
+  fd_vote_stake_weight_t const * stake_weights = fd_type_pun_const( msg+7UL );
   memcpy( gui->epoch.epochs[ idx ].stakes, stake_weights, staked_cnt*sizeof(fd_vote_stake_weight_t) );
 
   fd_epoch_leaders_delete( fd_epoch_leaders_leave( gui->epoch.epochs[ idx ].lsched ) );
@@ -1100,7 +1101,8 @@ fd_gui_handle_leader_schedule( fd_gui_t *    gui,
                                                                                staked_cnt,
                                                                                gui->epoch.epochs[ idx ].stakes,
                                                                                excluded_stake,
-                                                                               vote_keyed_lsched ) );
+                                                                               vote_keyed_lsched,
+                                                                               leader_slots ) );
 
   if( FD_UNLIKELY( start_slot==0UL ) ) {
     gui->epoch.epochs[ 0 ].start_time = fd_log_wallclock();
