@@ -283,6 +283,11 @@ unprivileged_init( fd_topo_t *      topo,
   }
 
   for( ulong i=0UL; i<(tile->in_cnt); i++ ) {
+    if (strcmp( topo->links[ tile->in_link_id[ i ] ].name, "poh_net_crds" ) == 0) {
+      ctx->in_kind[ i ] = IN_KIND_CRDS;
+      continue;
+    }
+
     if( !strstr( topo->links[ tile->in_link_id[ i ] ].name, "_net" ) ) {
       FD_LOG_ERR(( "in link %lu is not a net TX link", i ));
     }
@@ -290,6 +295,7 @@ unprivileged_init( fd_topo_t *      topo,
     ctx->link_tx[ i ].base   = topo->workspaces[ topo->objs[ link->dcache_obj_id ].wksp_id ].wksp;
     ctx->link_tx[ i ].chunk0 = fd_dcache_compact_chunk0( ctx->link_tx[ i ].base, link->dcache );
     ctx->link_tx[ i ].wmark  = fd_dcache_compact_wmark(  ctx->link_tx[ i ].base, link->dcache, link->mtu );
+    ctx->in_kind[ i ] = IN_KIND_TX;
   }
 
 }
@@ -525,6 +531,11 @@ before_frag( fd_sock_tile_t * ctx    FD_PARAM_UNUSED,
              ulong            in_idx FD_PARAM_UNUSED,
              ulong            seq    FD_PARAM_UNUSED,
              ulong            sig ) {
+  /* socket networking skips crds updates */
+  if ( FD_UNLIKELY(ctx->in_kind[ in_idx ] == IN_KIND_CRDS)) {
+    return 0;
+  }
+
   ulong proto = fd_disco_netmux_sig_proto( sig );
   if( FD_UNLIKELY( proto!=DST_PROTO_OUTGOING ) ) return 1;
   return 0; /* continue */
@@ -542,6 +553,11 @@ during_frag( fd_sock_tile_t * ctx,
              ulong            chunk,
              ulong            sz,
              ulong            ctl FD_PARAM_UNUSED ) {
+  /* socket networking skips crds updates */
+  if ( FD_UNLIKELY(ctx->in_kind[ in_idx ] == IN_KIND_CRDS)) {
+    return;
+  }
+
   if( FD_UNLIKELY( chunk<ctx->link_tx[ in_idx ].chunk0 || chunk>ctx->link_tx[ in_idx ].wmark || sz>FD_NET_MTU ) ) {
     FD_LOG_ERR(( "chunk %lu %lu corrupt, not in range [%lu,%lu]", chunk, sz, ctx->link_tx[ in_idx ].chunk0, ctx->link_tx[ in_idx ].wmark ));
   }
@@ -621,6 +637,10 @@ after_frag( fd_sock_tile_t *    ctx,
             ulong               tsorig FD_PARAM_UNUSED,
             ulong               tspub  FD_PARAM_UNUSED,
             fd_stem_context_t * stem   FD_PARAM_UNUSED ) {
+  /* socket networking skips crds updates */
+  if ( FD_UNLIKELY(ctx->in_kind[ in_idx ] == IN_KIND_CRDS)) {
+    return;
+  }
   /* Commit the packet added in during_frag */
 
   ctx->tx_idle_cnt = 0;
