@@ -25,6 +25,8 @@
 #include "../../../util/net/fd_ip4.h"
 #include "../../../util/net/fd_gre.h"
 #include "../../../util/fd_ip_wl.h"
+#include "../../../util/fd_staked_rate_limit.h"
+#include "../../fd_disco.h"
 
 #include <unistd.h>
 #include <linux/if.h> /* struct ifreq */
@@ -39,12 +41,18 @@
 
 #define MAX_NET_INS (32UL)
 
+#define IN_KIND_TX     (0UL)
+#define IN_KIND_CRDS   (1UL)
+
 /* FD_XDP_STATS_INTERVAL_NS controls the XDP stats refresh interval.
    This should be lower than the interval at which the metrics tile
    collects metrics. */
 
 #define FD_XDP_STATS_INTERVAL_NS (11e6) /* 11ms */
 
+/* Refresh time for rate limits */
+#define FD_RATE_LIMIT_REFRESH_INTERVAL_NS (1e6) /* 1ms, refresh takes roughly 2 micros */
+
 /* XSK_IDX_{MAIN,LO} are the hardcoded XSK indices in ctx->xsk[ ... ].
    Only net tile 0 has XSK_IDX_LO, all net tiles have XSK_IDX_MAIN. */
 
@@ -203,25 +211,34 @@ typedef struct {
 
   uint   default_address;
   uint   bind_address;
+  uint   host_ip_addr;
   ushort shred_listen_port;
   ushort quic_transaction_listen_port;
   ushort legacy_transaction_listen_port;
   ushort gossip_listen_port;
   ushort repair_intake_listen_port;
   ushort repair_serve_listen_port;
+  ushort repair_ancestor_hashes_listen_port;
   ushort send_src_port;
 
   uint ip_wl_enabled;
   fd_ip_wl ip_wl;
 
+  /* Rate limit refresh timer */
+  long rate_limit_refresh_interval_ticks;
+  long next_rate_limit_refresh;
+  fd_staked_rate_limit_t rate_limit;
+
   ulong in_cnt;
   fd_net_in_ctx_t in[ MAX_NET_INS ];
+  int             in_kind[ MAX_NET_INS ];
 
   fd_net_out_ctx_t quic_out[1];
   fd_net_out_ctx_t shred_out[1];
   fd_net_out_ctx_t gossip_out[1];
   fd_net_out_ctx_t repair_out[1];
   fd_net_out_ctx_t send_out[1];
+  fd_net_out_ctx_t netconn_out[1];
 
   /* XDP stats refresh timer */
   long xdp_stats_interval_ticks;
@@ -528,6 +545,12 @@ during_housekeeping( fd_net_ctx_t * ctx ) {
     ctx->next_xdp_stats_refresh = now + ctx->xdp_stats_interval_ticks;
     poll_xdp_statistics( ctx );
   }
+
+  /* refresh rate limits */
+  if (now > ctx->next_rate_limit_refresh) {
+    ctx->next_rate_limit_refresh = now + ctx->rate_limit_refresh_interval_ticks;
+    fd_staked_rate_limit_refresh(&ctx->rate_limit, now);
+  }
 }
 
 
@@ -629,6 +652,26 @@ net_tx_route( fd_net_ctx_t * ctx,
   return 1;
 }
 
+static inline void
+handle_new_cluster_contact_info( fd_net_ctx_t * ctx,
+                                 uchar const    * buf ) {
+  ulong const * header = (ulong const *)fd_type_pun_const( buf );
+
+  ulong dest_cnt = header[ 0 ];
+  uchar stakes_present = *(uchar const *)(header + 1UL);
+  uchar const * post_header_stake_flag = (uchar const *)(header + 1UL) + 1;
+  fd_shred_dest_wire_t const * in_dests = fd_type_pun_const( post_header_stake_flag );
+
+  FD_LOG_INFO(("fogo_fd_crds Receiving %lu contact infos, got stakes: %u", dest_cnt, stakes_present));
+  if (FD_UNLIKELY(!stakes_present)) {
+    FD_LOG_WARNING(("fogo_fd_crds No stakes present, skipping loading new IPs."));
+    return;
+  }
+
+  /* load new contact infos */
+  fd_staked_rate_limit_load_contacts(&ctx->rate_limit, in_dests, dest_cnt);
+}
+
 /* before_frag is called when a new metadata descriptor for a TX job is
    found.  This callback determines whether this net tile is responsible
    for the TX job.  If so, it prepares the TX op for the during_frag and
@@ -639,6 +682,11 @@ before_frag( fd_net_ctx_t * ctx,
              ulong          in_idx,
              ulong          seq,
              ulong          sig ) {
+  /* crds update from poh */
+  if ( FD_UNLIKELY( ctx->in_kind[ in_idx ] == IN_KIND_CRDS ) ) {
+    /* continue */
+    return 0;
+  }
   (void)in_idx; (void)seq;
 
   /* Find interface index of next packet */
@@ -731,6 +779,17 @@ during_frag( fd_net_ctx_t * ctx,
              ulong          chunk,
              ulong          sz,
              ulong          ctl FD_PARAM_UNUSED ) {
+  /* crds update from poh */
+  if ( FD_UNLIKELY( ctx->in_kind[ in_idx ] == IN_KIND_CRDS ) ) {
+    if( FD_UNLIKELY( chunk<ctx->in[ in_idx ].chunk0 || chunk>ctx->in[ in_idx ].wmark ) )
+      FD_LOG_ERR(( "crds chunk %lu %lu corrupt, not in range [%lu,%lu]", chunk, sz, ctx->in[ in_idx ].chunk0, ctx->in[ in_idx ].wmark ));
+
+    uchar const * dcache_entry = fd_chunk_to_laddr_const( ctx->in[ in_idx ].mem, chunk );
+    handle_new_cluster_contact_info( ctx, dcache_entry );
+    /* don't process as tx */
+    return;
+  }
+
   if( FD_UNLIKELY( chunk<ctx->in[ in_idx ].chunk0 || chunk>ctx->in[ in_idx ].wmark || sz>FD_NET_MTU ) )
     FD_LOG_ERR(( "chunk %lu %lu corrupt, not in range [%lu,%lu]", chunk, sz, ctx->in[ in_idx ].chunk0, ctx->in[ in_idx ].wmark ));
 
@@ -771,6 +830,11 @@ after_frag( fd_net_ctx_t *      ctx,
             ulong               tsorig,
             ulong               tspub,
             fd_stem_context_t * stem ) {
+  /* crds update from poh */
+  if ( FD_UNLIKELY( ctx->in_kind[ in_idx ] == IN_KIND_CRDS ) ) {
+    /* don't process as tx */
+    return;
+  }
   (void)in_idx; (void)seq; (void)sig; (void)tsorig; (void)tspub; (void)stem;
 
   /* Current send operation */
@@ -911,13 +975,15 @@ net_rx_packet( fd_net_ctx_t * ctx,
       ctx->metrics.rx_gre_ignored_cnt++; // drop. No gre interface in netdev table
       return;
     }
-    if( FD_UNLIKELY( FD_IP4_GET_VERSION( *iphdr )!=0x4 ) ) {
-      ctx->metrics.rx_gre_inv_pkt_cnt++; // drop. IP version!=IPv4
+    ulong gre_ipver = FD_IP4_GET_VERSION( *iphdr );
+    ulong gre_iplen = FD_IP4_GET_LEN( *iphdr );
+    if( FD_UNLIKELY( gre_ipver!=0x4 || gre_iplen<20 ) ) {
+      FD_DTRACE_PROBE( net_tile_err_rx_noip );
+      ctx->metrics.rx_gre_inv_pkt_cnt++; /* drop IPv6 packets */
       return;
     }
 
-    ulong overhead = FD_IP4_GET_LEN( *iphdr ) + sizeof(fd_gre_hdr_t);
-
+    ulong overhead = gre_iplen + sizeof(fd_gre_hdr_t);
     if( FD_UNLIKELY( (uchar *)iphdr+overhead+sizeof(fd_ip4_hdr_t)>packet_end ) ) {
       FD_DTRACE_PROBE( net_tile_err_rx_undersz );
       ctx->metrics.rx_undersz_cnt++;  // inner ip4 header invalid
@@ -939,13 +1005,16 @@ net_rx_packet( fd_net_ctx_t * ctx,
   ulong ctl         = umem_off & 0x3fUL;
 
   /* Filter for UDP/IPv4 packets. */
-  if( FD_UNLIKELY( ( FD_IP4_GET_VERSION( *iphdr )!=0x4 ) ||
-                   ( iphdr->protocol!=FD_IP4_HDR_PROTOCOL_UDP ) ) ) return;
+  ulong ipver = FD_IP4_GET_VERSION( *iphdr );
+  ulong iplen = FD_IP4_GET_LEN    ( *iphdr );
+  if( FD_UNLIKELY( ipver!=0x4 || iplen<20 ||
+                   iphdr->protocol!=FD_IP4_HDR_PROTOCOL_UDP ) ) {
+    FD_DTRACE_PROBE( net_tile_err_rx_noip );
+    ctx->metrics.rx_undersz_cnt++; /* drop IPv6 packets */
+    return;
+  }
 
-  /* IPv4 is variable-length, so lookup IHL to find start of UDP */
-  uint iplen        = FD_IP4_GET_LEN( *iphdr );
   uchar const * udp = (uchar *)iphdr + iplen;
-
   if( FD_UNLIKELY( udp+sizeof(fd_udp_hdr_t) > packet_end ) ) {
     FD_DTRACE_PROBE( net_tile_err_rx_undersz );
     ctx->metrics.rx_undersz_cnt++;
@@ -982,6 +1051,9 @@ net_rx_packet( fd_net_ctx_t * ctx,
   } else if( FD_UNLIKELY( udp_dstport==ctx->repair_serve_listen_port ) ) {
     proto = DST_PROTO_REPAIR;
     out = ctx->repair_out;
+  } else if( FD_UNLIKELY( udp_dstport==ctx->repair_ancestor_hashes_listen_port ) ) {
+    proto = DST_PROTO_ANCESTOR;
+    out = ctx->repair_out;
   } else if( FD_UNLIKELY( udp_dstport==ctx->send_src_port ) ) {
     proto = DST_PROTO_SEND;
     out = ctx->send_out;
@@ -1006,6 +1078,36 @@ net_rx_packet( fd_net_ctx_t * ctx,
     return;
   }
 
+  /* Rate limit globally and per staked IP (skip traffic from self) */
+  if( FD_LIKELY( ip_srcaddr != ctx->host_ip_addr ) ) {
+    if( FD_UNLIKELY( !fd_staked_rate_limit_check( &ctx->rate_limit, ip_srcaddr ) ) ) {
+      ctx->metrics.packet_ip_blocked_cnt++;
+      return;
+    }
+  }
+
+  /* For protocols that are supported by netcon, check if we should route to it
+     If netconn_out is configured and gossip_out/repair_out (pure FD) are not, route to netconn. */
+  if( FD_UNLIKELY( ctx->netconn_out->mcache != NULL ) ) {
+    if( proto == DST_PROTO_GOSSIP && ctx->gossip_out->mcache == NULL ) {
+      int is_staked = (ip_srcaddr == ctx->host_ip_addr) ||
+                      fd_staked_rate_limit_contains_ip( &ctx->rate_limit, ip_srcaddr );
+      proto = (ushort)(is_staked ? DST_PROTO_NETCONN_GOSSIP_STAKED : DST_PROTO_NETCONN_GOSSIP_UNSTAKED);
+      out = ctx->netconn_out;
+    } else if( proto == DST_PROTO_REPAIR && ctx->repair_out->mcache == NULL ) {
+      int is_staked = (ip_srcaddr == ctx->host_ip_addr) ||
+                      fd_staked_rate_limit_contains_ip( &ctx->rate_limit, ip_srcaddr );
+      proto = (ushort)(is_staked ? DST_PROTO_NETCONN_REPAIR_STAKED : DST_PROTO_NETCONN_REPAIR_UNSTAKED);
+      out = ctx->netconn_out;
+    } else if( proto == DST_PROTO_ANCESTOR ) {
+      FD_LOG_NOTICE(("fd_netconn ancestor %u", proto));
+      int is_staked = (ip_srcaddr == ctx->host_ip_addr) ||
+                      fd_staked_rate_limit_contains_ip( &ctx->rate_limit, ip_srcaddr );
+      proto = (ushort)(is_staked ? DST_PROTO_NETCONN_ANCESTOR_STAKED : DST_PROTO_NETCONN_ANCESTOR_UNSTAKED);
+      out = ctx->netconn_out;
+    }
+  }
+
   /* tile can decide how to partition based on src ip addr and src port */
   ulong sig              = fd_disco_netmux_sig( ip_srcaddr, udp_srcport, ip_srcaddr, proto, 14UL+8UL+iplen );
 
@@ -1328,6 +1430,7 @@ privileged_init( fd_topo_t *      topo,
       (ushort)tile->xdp.net.gossip_listen_port,
       (ushort)tile->xdp.net.repair_intake_listen_port,
       (ushort)tile->xdp.net.repair_serve_listen_port,
+      (ushort)tile->xdp.net.repair_ancestor_hashes_listen_port,
       (ushort)tile->xdp.net.send_src_port
     };
 
@@ -1406,7 +1509,9 @@ unprivileged_init( fd_topo_t *      topo,
   ctx->gossip_listen_port             = tile->net.gossip_listen_port;
   ctx->repair_intake_listen_port      = tile->net.repair_intake_listen_port;
   ctx->repair_serve_listen_port       = tile->net.repair_serve_listen_port;
+  ctx->repair_ancestor_hashes_listen_port       = tile->net.repair_ancestor_hashes_listen_port;
   ctx->send_src_port                  = tile->net.send_src_port;
+  ctx->host_ip_addr                   = tile->net.host_ip_addr;
 
   // setup ip whitelist
   ctx->ip_wl_enabled = tile->net.ip_whitelist_cnt != 0;
@@ -1422,6 +1527,21 @@ unprivileged_init( fd_topo_t *      topo,
     FD_LOG_NOTICE(( "IP whitelisting in the XDP tile is disabled." ));
   }
 
+  /* rate limits configuration */
+  FD_LOG_NOTICE(("Rate limits configured as unstaked %upps, staked ip: %upps", tile->net.unstaked_packet_rate_limit, tile->net.staked_ip_packet_rate_limit));
+  /* Set the burst the twice the rate limit */
+  double tick_per_ns = fd_tempo_tick_per_ns( NULL );
+  fd_staked_rate_limit_init(
+    &ctx->rate_limit,
+    tile->net.unstaked_packet_rate_limit,
+    2 * tile->net.unstaked_packet_rate_limit,
+    tile->net.staked_ip_packet_rate_limit,
+    2 * tile->net.staked_ip_packet_rate_limit,
+    tick_per_ns,
+    fd_tickcount()
+  );
+  ctx->rate_limit_refresh_interval_ticks = (long)( FD_RATE_LIMIT_REFRESH_INTERVAL_NS * tick_per_ns );
+
   /* Put a bound on chunks we read from the input, to make sure they
      are within in the data region of the workspace. */
 
@@ -1430,7 +1550,14 @@ unprivileged_init( fd_topo_t *      topo,
   FD_TEST( tile->in_cnt<=32 );
   for( ulong i=0UL; i<tile->in_cnt; i++ ) {
     fd_topo_link_t * link = &topo->links[ tile->in_link_id[ i ] ];
-    if( FD_UNLIKELY( link->mtu!=FD_NET_MTU ) ) FD_LOG_ERR(( "net tile in link %s does not have a normal MTU", link->name ));
+
+    /* incoming links are either crds updates or tx packets */
+    if (strcmp( topo->links[ tile->in_link_id[ i ] ].name, "poh_net_crds" ) == 0) {
+      ctx->in_kind[ i ] = IN_KIND_CRDS;
+    } else {
+      if( FD_UNLIKELY( link->mtu!=FD_NET_MTU ) ) FD_LOG_ERR(( "net tile in link %s does not have a normal MTU", link->name ));
+      ctx->in_kind[ i ] = IN_KIND_TX;
+    }
 
     ctx->in[ i ].mem    = topo->workspaces[ topo->objs[ link->dcache_obj_id ].wksp_id ].wksp;
     ctx->in[ i ].chunk0 = fd_dcache_compact_chunk0( ctx->in[ i ].mem, link->dcache );
@@ -1474,24 +1601,34 @@ unprivileged_init( fd_topo_t *      topo,
       ctx->send_out->sync   = fd_mcache_seq_laddr( ctx->send_out->mcache );
       ctx->send_out->depth  = fd_mcache_depth( ctx->send_out->mcache );
       ctx->send_out->seq    = fd_mcache_seq_query( ctx->send_out->sync );
+    } else if( strcmp( out_link->name, "net_netcon" ) == 0 ) {
+      fd_topo_link_t * netconn_out = out_link;
+      ctx->netconn_out->mcache = netconn_out->mcache;
+      ctx->netconn_out->sync   = fd_mcache_seq_laddr( ctx->netconn_out->mcache );
+      ctx->netconn_out->depth  = fd_mcache_depth( ctx->netconn_out->mcache );
+      ctx->netconn_out->seq    = fd_mcache_seq_query( ctx->netconn_out->sync );
     } else {
       FD_LOG_ERR(( "unrecognized out link `%s`", out_link->name ));
     }
   }
 
-  /* Check if any of the tiles we set a listen port for do not have an outlink. */
+  /* Check if any of the tiles we set a listen port for do not have an outlink.
+     For gossip/repair, either the native out link OR netconn_out must be configured. */
+  int has_netconn = ctx->netconn_out->mcache != NULL;
   if( FD_UNLIKELY( ctx->shred_listen_port!=0 && ctx->shred_out->mcache==NULL ) ) {
     FD_LOG_ERR(( "shred listen port set but no out link was found" ));
   } else if( FD_UNLIKELY( ctx->quic_transaction_listen_port!=0 && ctx->quic_out->mcache==NULL ) ) {
     FD_LOG_ERR(( "quic transaction listen port set but no out link was found" ));
   } else if( FD_UNLIKELY( ctx->legacy_transaction_listen_port!=0 && ctx->quic_out->mcache==NULL ) ) {
     FD_LOG_ERR(( "legacy transaction listen port set but no out link was found" ));
-  } else if( FD_UNLIKELY( ctx->gossip_listen_port!=0 && ctx->gossip_out->mcache==NULL ) ) {
-    FD_LOG_ERR(( "gossip listen port set but no out link was found" ));
+  } else if( FD_UNLIKELY( ctx->gossip_listen_port!=0 && ctx->gossip_out->mcache==NULL && !has_netconn ) ) {
+    FD_LOG_ERR(( "gossip listen port set but no out link (gossip or netconn) was found" ));
   } else if( FD_UNLIKELY( ctx->repair_intake_listen_port!=0 && ctx->repair_out->mcache==NULL ) ) {
     FD_LOG_ERR(( "repair intake port set but no out link was found" ));
-  } else if( FD_UNLIKELY( ctx->repair_serve_listen_port!=0 && ctx->repair_out->mcache==NULL ) ) {
-    FD_LOG_ERR(( "repair serve listen port set but no out link was found" ));
+  } else if( FD_UNLIKELY( ctx->repair_serve_listen_port!=0 && ctx->repair_out->mcache==NULL && !has_netconn ) ) {
+    FD_LOG_ERR(( "repair serve listen port set but no out link (repair or netconn) was found" ));
+  } else if( FD_UNLIKELY( ctx->repair_ancestor_hashes_listen_port!=0 && !has_netconn ) ) {
+    FD_LOG_ERR(( "repair ancestor hashes listen port set but no out link netconn was found" ));
   } else if( FD_UNLIKELY( ctx->neigh4_solicit->mcache==NULL ) ) {
     FD_LOG_ERR(( "netlink request link not found" ));
   } else if( FD_UNLIKELY( ctx->send_src_port!=0 && ctx->send_out->mcache==NULL ) ) {
