@@ -117,7 +117,22 @@ struct fd_pack_limits {
      fd_pack_end_block is called. */
   ulong max_txn_per_microblock;      /* in [0, 16777216] */
   ulong max_microblocks_per_block;   /* in [0, 1e12) */
-  ulong max_txn_per_block; // Fogo change
+
+  // In Fogo we limit the number of transactions per block.
+  // Every txn consumes CU, but also has verification overhead.
+  // Too many small txns can overhelm the validator.
+  // Only a block packed with tiny 150 CU txns should hit this limit in practice.
+  // This limit is set to a value that will not overwhelm the validator.
+  // The exact limit is set by agave.
+  ulong max_txn_per_block;
+
+  // In Fogo we also limit the number txn paylaod bytes per block.
+  // We do this because every payload byte puts pressure on the store
+  // tile, eg rocksdb.
+  // This limit is set to an empirically derived safe value, the validator
+  // can handle.
+  // The exact limit is set by agave.
+  ulong max_txn_bytes_per_block;
 
 };
 typedef struct fd_pack_limits fd_pack_limits_t;
@@ -189,7 +204,7 @@ fd_pack_t * fd_pack_join( void * mem );
 
 /* For performance reasons, implement this here.  The offset is STATIC_ASSERTed
    in fd_pack.c. */
-#define FD_PACK_PENDING_TXN_CNT_OFF 80
+#define FD_PACK_PENDING_TXN_CNT_OFF 88
 FD_FN_PURE static inline ulong
 fd_pack_avail_txn_cnt( fd_pack_t const * pack ) {
   return *((ulong const *)((uchar const *)pack + FD_PACK_PENDING_TXN_CNT_OFF));
