@@ -166,6 +166,7 @@ typedef struct {
     ulong slot_max_vote_cost;
     ulong slot_max_write_cost_per_acct;
     ulong txn_limit;
+    ulong txn_byte_limit;
   } limits;
 
   /* If drain_banks is non-zero, then the pack tile must wait until all
@@ -975,6 +976,7 @@ after_frag( fd_pack_ctx_t *     ctx,
     ctx->limits.slot_max_vote_cost           = ctx->_became_leader->limits.slot_max_vote_cost;
     ctx->limits.slot_max_write_cost_per_acct = ctx->_became_leader->limits.slot_max_write_cost_per_acct;
     ctx->limits.txn_limit                    = ctx->_became_leader->limits.txn_limit;
+    ctx->limits.txn_byte_limit               = ctx->_became_leader->limits.txn_byte_limit;
 
     /* ticks_per_ns is probably relatively stable over 400ms, but not
        over several hours, so we need to compute the slot duration in
@@ -1013,6 +1015,7 @@ after_frag( fd_pack_ctx_t *     ctx,
     limits->max_write_cost_per_acct = ctx->limits.slot_max_write_cost_per_acct;
     limits->max_txn_per_microblock = ULONG_MAX; /* unused */
     limits->max_txn_per_block = ctx->limits.txn_limit;
+    limits->max_txn_bytes_per_block = ctx->limits.txn_byte_limit;
     fd_pack_set_block_limits( ctx->pack, limits );
     fd_pack_pacing_update_consumed_cus( ctx->pacer, fd_pack_current_block_cost( ctx->pack ), now );
 
