@@ -39,8 +39,8 @@ fd_stake_ci_new( void             * mem,
     ei->slot_cnt       = 0UL;
     ei->excluded_stake = 0UL;
     ei->vote_keyed_lsched = 0UL;
-
-    ei->lsched = fd_epoch_leaders_join( fd_epoch_leaders_new( ei->_lsched, 0UL, 0UL, 1UL, 1UL,    info->vote_stake_weight,  0UL, ei->vote_keyed_lsched ) );
+    ei->leader_slots     = 1500UL; // FOGO: dummy value, will be overridden in fd_stake_ci_stake_msg_init
+    ei->lsched = fd_epoch_leaders_join( fd_epoch_leaders_new( ei->_lsched, 0UL, 0UL, 1UL, 1UL,    info->vote_stake_weight,  0UL, ei->vote_keyed_lsched, ei->leader_slots ) );
     ei->sdest  = fd_shred_dest_join   ( fd_shred_dest_new   ( ei->_sdest,  info->shred_dest, 1UL, ei->lsched, identity_key, 0UL ) );
   }
   info->identity_key[ 0 ] = *identity_key;
@@ -67,6 +67,7 @@ fd_stake_ci_stake_msg_init( fd_stake_ci_t               * info,
   info->scratch->staked_cnt     = msg->staked_cnt;
   info->scratch->excluded_stake = msg->excluded_stake;
   info->scratch->vote_keyed_lsched = msg->vote_keyed_lsched;
+  info->scratch->leader_slots     = msg->leader_slots;
 
   fd_memcpy( info->vote_stake_weight, msg->weights, msg->staked_cnt*sizeof(fd_vote_stake_weight_t) );
 }
@@ -138,6 +139,7 @@ fd_stake_ci_stake_msg_fini( fd_stake_ci_t * info ) {
   ulong staked_cnt             = info->scratch->staked_cnt;
   ulong unchanged_staked_cnt   = info->scratch->staked_cnt;
   ulong vote_keyed_lsched      = info->scratch->vote_keyed_lsched;
+  ulong leader_slots           = info->scratch->leader_slots;
 
   /* Just take the first one arbitrarily because they both have the same
      contact info, other than possibly some staked nodes with no contact
@@ -210,9 +212,10 @@ fd_stake_ci_stake_msg_fini( fd_stake_ci_t * info ) {
   new_ei->slot_cnt       = info->scratch->slot_cnt;
   new_ei->excluded_stake = excluded_stake;
   new_ei->vote_keyed_lsched = vote_keyed_lsched;
+  new_ei->leader_slots     = leader_slots;
 
   new_ei->lsched = fd_epoch_leaders_join( fd_epoch_leaders_new( new_ei->_lsched, epoch, new_ei->start_slot, new_ei->slot_cnt,
-                                                                unchanged_staked_cnt, info->vote_stake_weight, excluded_stake, vote_keyed_lsched ) );
+                                                                unchanged_staked_cnt, info->vote_stake_weight, excluded_stake, vote_keyed_lsched, leader_slots ) );
   new_ei->sdest  = fd_shred_dest_join   ( fd_shred_dest_new   ( new_ei->_sdest, info->shred_dest, j,
                                                                 new_ei->lsched, info->identity_key,  excluded_stake ) );
   log_summary( "stake update", info );
