@@ -100,10 +100,16 @@ struct fd_topo_net_tile {
   ushort gossip_listen_port;
   ushort repair_intake_listen_port;
   ushort repair_serve_listen_port;
+  ushort repair_ancestor_hashes_listen_port;
   ushort send_src_port;
 
   uint ip_whitelist_cnt;
   uint ip_whitelist[ 512UL ];
+
+  uint unstaked_packet_rate_limit;
+  uint staked_ip_packet_rate_limit;
+
+  uint host_ip_addr;
 };
 
 typedef struct fd_topo_net_tile fd_topo_net_tile_t;
